@@ -81,7 +81,7 @@
 #' gpg <- rpost(n = 1000, model = "gp", prior = fp, thresh = u, data = gom)
 #' plot(gpg)
 #'
-#' \dontrun{
+#' \donttest{
 #' # Using the bayesplot package
 #' plot(gpg, use_bayesplot = TRUE)
 #' plot(gpg, use_bayesplot = TRUE, pars = "xi", prob = 0.95)
@@ -103,7 +103,7 @@
 #' plot(bgpg, pu_only = TRUE)
 #' plot(bgpg, add_pu = TRUE)
 #'
-#' \dontrun{
+#' \donttest{
 #' # Using the bayesplot package
 #' dimnames(bgpg$bin_sim_vals)
 #' plot(bgpg, use_bayesplot = TRUE)
@@ -236,7 +236,9 @@ plot.evpost <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
     zz[zz == -Inf] <- NA
     dx <- diff(xx[1:2]); dy <- diff(yy[1:2]); sz <- sort(zz)
     c1 <- cumsum(sz) * dx * dy; c1 <- c1/max(c1)
-    con_levs <- sapply(prob, function(x) stats::approx(c1, sz, xout = 1 - x)$y)
+    # Suppress a warning that comes from stats:::regularize.values
+    con_levs <- suppressWarnings(sapply(prob, function(x)
+      stats::approx(c1, sz, xout = 1 - x)$y))
     #
     graphics::contour(xx, yy, zz, levels = con_levs, add = FALSE, ann = FALSE,
       labels = prob * 100, ...)
@@ -274,8 +276,8 @@ plot.evpost <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
         ylabs <- rep(NA, x$d)
       }
     }
-    def.par <- graphics::par(no.readonly = TRUE)
-    graphics::par(mfrow = c(rows, cols))
+    oldpar <- graphics::par(mfrow = c(rows, cols))
+    on.exit(graphics::par(oldpar))
     pairwise_plots <- function(x) {
       for (i in 1:(ncol(x)-1)) {
         for (j in (i+1):ncol(x)) {
@@ -286,7 +288,6 @@ plot.evpost <- function(x, y, ..., n = ifelse(x$d == 1, 1001, 101),
       }
     }
     pairwise_plots(plot_data)
-    graphics::par(def.par)
   }
 }
 
